==> pakfire-0.9.28.tar.gz: pakfire-0.9.28/src/_pakfire/file.c <==
/*#############################################################################
#                                                                             #
# Pakfire - The IPFire package management system                              #
# Copyright (C) 2022 Pakfire development team                                 #
#                                                                             #
# This program is free software: you can redistribute it and/or modify        #
# it under the terms of the GNU General Public License as published by        #
# the Free Software Foundation, either version 3 of the License, or           #
# (at your option) any later version.                                         #
#                                                                             #
# This program is distributed in the hope that it will be useful,             #
# but WITHOUT ANY WARRANTY; without even the implied warranty of              #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               #
# GNU General Public License for more details.                                #
#                                                                             #
# You should have received a copy of the GNU General Public License           #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.       #
#                                                                             #
#############################################################################*/

#include <Python.h>
#include <datetime.h>

#include <pakfire/digest.h>
#include <pakfire/file.h>

#include "file.h"
#include "pakfire.h"
#include "util.h"

PyObject* new_file(struct pakfire_file* file) {
	PyTypeObject* type = &FileType;

	FileObject* self = (FileObject *)type->tp_alloc(type, 0);
	if (self) {
		if (file)
			self->file = pakfire_file_ref(file);
		else
			self->file = NULL;
	}

	return (PyObject *)self;
}

static PyObject* File_new(PyTypeObject* type, PyObject* args, PyObject* kwds) {
	return new_file(NULL);
}

static void File_dealloc(FileObject* self) {
	if (self->file)
		pakfire_file_unref(self->file);

	Py_TYPE(self)->tp_free((PyObject *)self);
}

static int File_init(FileObject* self, PyObject* args, PyObject* kwds) {
	PakfireObject* pakfire = NULL;

	if (!PyArg_ParseTuple(args, "O!", &PakfireType, &pakfire))
		return -1;

	// Create a new File object
	int r = pakfire_file_create(&self->file, pakfire->pakfire);
	if (r)
		return -1;

	return 0;
}

static PyObject* File_repr(FileObject* self) {
	const char* path = pakfire_file_get_path(self->file);

	return PyUnicode_FromFormat("<_pakfire.Problem %s>", path);
}

static PyObject* File_get_path(FileObject* self) {
	const char* path = pakfire_file_get_path(self->file);

	return PyUnicode_FromString(path);
}

static int File_set_path(FileObject* self, PyObject* value) {
	const char* path = PyUnicode_AsUTF8(value);
	if (!path)
		return -1;

	pakfire_file_set_path(self->file, path);
	return 0;
}

static PyObject* File_get_size(FileObject* self) {
	const size_t size = pakfire_file_get_size(self->file);

	return PyLong_FromSize_t(size);
}

static int File_set_size(FileObject* self, PyObject* value) {
	long size = PyLong_AsLong(value);
	if (size < 0)
		return -1;

	pakfire_file_set_size(self->file, size);
	return 0;
}

static PyObject* File_get_type(FileObject* self) {
	int type = pakfire_file_get_type(self->file);

	return PyLong_FromLong(type);
}

static PyObject* File_get_uname(FileObject* self) {
	const char* uname = pakfire_file_get_uname(self->file);

	return PyUnicode_FromString(uname);
}

static int File_set_uname(FileObject* self, PyObject* value) {
	const char* uname = PyUnicode_AsUTF8(value);
	if (!uname)
		return -1;

	pakfire_file_set_uname(self->file, uname);
	return 0;
}

static PyObject* File_get_gname(FileObject* self) {
	const char* gname = pakfire_file_get_gname(self->file);

	return PyUnicode_FromString(gname);
}

static int File_set_gname(FileObject* self, PyObject* value) {
	const char* gname = PyUnicode_AsUTF8(value);
	if (!gname)
		return -1;

	pakfire_file_set_gname(self->file, gname);
	return 0;
}

static PyObject* File_get_mode(FileObject* self) {
	const mode_t mode = pakfire_file_get_mode(self->file);

	return PyLong_FromLong(mode);
}

static int File_set_mode(FileObject* self, PyObject* value) {
	const mode_t mode = PyLong_AsLong(value);

	pakfire_file_set_mode(self->file, mode);
	return 0;
}

static time_t PyDateTime_AsTime_t(PyObject* value) {
	if (!PyDateTime_Check(value))
		return -1;

	PyDateTime_IMPORT;

	// Convert from datetime to struct tm
	struct tm tm = {
		.tm_year  = PyDateTime_GET_YEAR(value) - 1900,
		.tm_mon   = PyDateTime_GET_MONTH(value) - 1,
		.tm_mday  = PyDateTime_GET_DAY(value),
		.tm_hour  = PyDateTime_DATE_GET_HOUR(value),
		.tm_min   = PyDateTime_DATE_GET_MINUTE(value),
		.tm_sec   = PyDateTime_DATE_GET_SECOND(value),
		.tm_isdst = 0,
	};

	// Convert into time_t
	const time_t t = mktime(&tm);
	if (t < 0) {
		PyErr_SetFromErrno(PyExc_OSError);
		return -1;
	}

	return t;
}

static PyObject* File_get_ctime(FileObject* self) {
	const time_t t = pakfire_file_get_ctime(self->file);

	return PyDateTime_FromTime_t(&t);
}

static int File_set_ctime(FileObject* self, PyObject* value) {
	const time_t t = PyDateTime_AsTime_t(value);
	if (t < 0)
		return -1;

	pakfire_file_set_ctime(self->file, t);
	return 0;
}

static PyObject* File_get_mtime(FileObject* self) {
	const time_t t = pakfire_file_get_mtime(self->file);

	return PyDateTime_FromTime_t(&t);
}

static int File_set_mtime(FileObject* self, PyObject* value) {
	const time_t t = PyDateTime_AsTime_t(value);
	if (t < 0)
		return -1;

	pakfire_file_set_mtime(self->file, t);
	return 0;
}

static PyObject* File_digest(FileObject* self, PyObject* args) {
	const char* name = NULL;

	if (!PyArg_ParseTuple(args, "s", &name))
		return NULL;

	// Fetch the type
	const enum pakfire_digest_types type = pakfire_digest_get_by_name(name);

	// Raise ValueError if we could not find the type
	if (!type) {
		PyErr_Format(PyExc_ValueError, "Unknown digest type: %s", name);
		return NULL;
	}

	size_t length = 0;

	// Fetch the digest
	const unsigned char* digest = pakfire_file_get_digest(self->file, type, &length);
	if (!digest)
		Py_RETURN_NONE;

	return PyBytes_FromStringAndSize((const char*)digest, length);
}

static struct PyMethodDef File_methods[] = {
	{
		"digest",
		(PyCFunction)File_digest,
		METH_VARARGS,
		NULL,
	},
	{ NULL },
};

static struct PyGetSetDef File_getsetters[] = {
	{
		"ctime",
		(getter)File_get_ctime,
		(setter)File_set_ctime,
		NULL,
		NULL,
	},
	{
		"gname",
		(getter)File_get_gname,
		(setter)File_set_gname,
		NULL,
		NULL,
	},
	{
		"mode",
		(getter)File_get_mode,
		(setter)File_set_mode,
		NULL,
		NULL,
	},
	{
		"mtime",
		(getter)File_get_mtime,
		(setter)File_set_mtime,
		NULL,
		NULL,
	},
	{
		"path",
		(getter)File_get_path,
		(setter)File_set_path,
		NULL,
		NULL,
	},
	{
		"size",
		(getter)File_get_size,
		(setter)File_set_size,
		NULL,
		NULL,
	},
	{
		"type",
		(getter)File_get_type,
		NULL,
		NULL,
		NULL,
	},
	{
		"uname",
		(getter)File_get_uname,
		(setter)File_set_uname,
		NULL,
		NULL,
	},
	{ NULL },
};

PyTypeObject FileType = {
	PyVarObject_HEAD_INIT(NULL, 0)
	tp_name:            "_pakfire.File",
	tp_basicsize:       sizeof(FileObject),
	tp_flags:           Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE,
	tp_new:             File_new,
	tp_dealloc:         (destructor)File_dealloc,
	tp_init:            (initproc)File_init,
	tp_doc:             "File object",
	tp_methods:         File_methods,
	tp_getset:          File_getsetters,
	tp_repr:            (reprfunc)File_repr,
	tp_str:             (reprfunc)File_get_path,
};

==> pakfire-0.9.29.tar.gz: pakfire-0.9.29/src/_pakfire/file.c <==
/*#############################################################################
#                                                                             #
# Pakfire - The IPFire package management system                              #
# Copyright (C) 2022 Pakfire development team                                 #
#                                                                             #
# This program is free software: you can redistribute it and/or modify        #
# it under the terms of the GNU General Public License as published by        #
# the Free Software Foundation, either version 3 of the License, or           #
# (at your option) any later version.                                         #
#                                                                             #
# This program is distributed in the hope that it will be useful,             #
# but WITHOUT ANY WARRANTY; without even the implied warranty of              #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               #
# GNU General Public License for more details.                                #
#                                                                             #
# You should have received a copy of the GNU General Public License           #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.       #
#                                                                             #
#############################################################################*/

#define PY_SSIZE_T_CLEAN
#include <Python.h>

#include <pakfire/digest.h>
#include <pakfire/file.h>

#include "file.h"
#include "pakfire.h"
#include "util.h"

PyObject* new_file(struct pakfire_file* file) {
	PyTypeObject* type = &FileType;

	FileObject* self = (FileObject *)type->tp_alloc(type, 0);
	if (self) {
		self->file = pakfire_file_ref(file);
	}

	return (PyObject *)self;
}

static void File_dealloc(FileObject* self) {
	if (self->file)
		pakfire_file_unref(self->file);

	Py_TYPE(self)->tp_free((PyObject *)self);
}

static PyObject* File_repr(FileObject* self) {
	const char* path = pakfire_file_get_path(self->file);

	return PyUnicode_FromFormat("<_pakfire.File %s>", path);
}

static PyObject* File_get_path(FileObject* self) {
	const char* path = pakfire_file_get_path(self->file);
	if (!path)
		Py_RETURN_NONE;

	return PyUnicode_FromString(path);
}

static PyObject* File_get_size(FileObject* self) {
	const size_t size = pakfire_file_get_size(self->file);

	return PyLong_FromSize_t(size);
}

static PyObject* File_get_type(FileObject* self) {
	int type = pakfire_file_get_type(self->file);

	return PyLong_FromLong(type);
}

static PyObject* File_get_uname(FileObject* self) {
	const char* uname = pakfire_file_get_uname(self->file);
	if (!uname)
		Py_RETURN_NONE;

	return PyUnicode_FromString(uname);
}

static PyObject* File_get_gname(FileObject* self) {
	const char* gname = pakfire_file_get_gname(self->file);
	if (!gname)
		Py_RETURN_NONE;

	return PyUnicode_FromString(gname);
}

static PyObject* File_get_mode(FileObject* self) {
	const mode_t mode = pakfire_file_get_mode(self->file);

	return PyLong_FromLong(mode);
}

static PyObject* File_get_ctime(FileObject* self) {
	const time_t t = pakfire_file_get_ctime(self->file);

	return PyDateTime_FromTime_t(&t);
}

static PyObject* File_get_mtime(FileObject* self) {
	const time_t t = pakfire_file_get_mtime(self->file);

	return PyDateTime_FromTime_t(&t);
}

static PyObject* File_digest(FileObject* self, PyObject* args) {
	const char* name = NULL;

	if (!PyArg_ParseTuple(args, "s", &name))
		return NULL;

	// Fetch the type
	const enum pakfire_digest_types type = pakfire_digest_get_by_name(name);

	// Raise ValueError if we could not find the type
	if (!type) {
		PyErr_Format(PyExc_ValueError, "Unknown digest type: %s", name);
		return NULL;
	}

	size_t length = 0;

	// Fetch the digest
	const unsigned char* digest = pakfire_file_get_digest(self->file, type, &length);
	if (!digest)
		Py_RETURN_NONE;

	return PyBytes_FromStringAndSize((const char*)digest, length);
}

static PyObject* File_get_mimetype(FileObject* self) {
	const char* mimetype = pakfire_file_get_mimetype(self->file);
	if (!mimetype)
		Py_RETURN_NONE;

	return PyUnicode_FromString(mimetype);
}

static PyObject* File_get_capabilities(FileObject* self) {
	PyObject* result = NULL;
	char* caps = NULL;

	// No capabilities
	if (!pakfire_file_has_caps(self->file))
		Py_RETURN_NONE;

	// Fetch capabilities
	caps = pakfire_file_get_caps(self->file);
	if (!caps) {
		PyErr_SetFromErrno(PyExc_OSError);
		goto ERROR;
	}

	// Convert to Python string object
	result = PyUnicode_FromString(caps);

ERROR:
	if (caps)
		free(caps);

	return result;
}

static struct PyMethodDef File_methods[] = {
	{
		"digest",
		(PyCFunction)File_digest,
		METH_VARARGS,
		NULL,
	},
	{ NULL },
};

static struct PyGetSetDef File_getsetters[] = {
	{
		"capabilities",
		(getter)File_get_capabilities,
		NULL,
		NULL,
		NULL,
	},
	{
		"ctime",
		(getter)File_get_ctime,
		NULL,
		NULL,
		NULL,
	},
	{
		"gname",
		(getter)File_get_gname,
		NULL,
		NULL,
		NULL,
	},
	{
		"mimetype",
		(getter)File_get_mimetype,
		NULL,
		NULL,
		NULL,
	},
	{
		"mode",
		(getter)File_get_mode,
		NULL,
		NULL,
		NULL,
	},
	{
		"mtime",
		(getter)File_get_mtime,
		NULL,
		NULL,
		NULL,
	},
	{
		"path",
		(getter)File_get_path,
		NULL,
		NULL,
		NULL,
	},
	{
		"size",
		(getter)File_get_size,
		NULL,
		NULL,
		NULL,
	},
	{
		"type",
		(getter)File_get_type,
		NULL,
		NULL,
		NULL,
	},
	{
		"uname",
		(getter)File_get_uname,
		NULL,
		NULL,
		NULL,
	},
	{ NULL },
};

PyTypeObject FileType = {
	PyVarObject_HEAD_INIT(NULL, 0)
	tp_name:            "_pakfire.File",
	tp_basicsize:       sizeof(FileObject),
	tp_flags:           Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE,
	tp_dealloc:         (destructor)File_dealloc,
	tp_doc:             "File object",
	tp_methods:         File_methods,
	tp_getset:          File_getsetters,
	tp_repr:            (reprfunc)File_repr,
	tp_str:             (reprfunc)File_get_path,
};
